import urllib.request
import urllib.error
import errno
import shutil, sys
from io import BytesIO
from os import remove, chdir, mkdir
from os.path import exists
from zipfile import ZipFile


class paqAdm():
    def getlibrary(self, actual_dir, name, liburl, version='latest'):
        """Metodo para manejar la descarga del paquete comprimido, su
        descompresion y ubicación"""

        # Necesario para acceder al espacio release de github
        if version == 'latest':
            aux = '/releases/latest/download'
            url = urllib.request.Request(f"{liburl}{aux}/{name}.zip")
        else:
            aux = '/releases/download/'

            # Conformando la petición con todas las variantes.
            url = urllib.request.Request(f"{liburl}{aux}{version}/{name}.zip")

        if not exists('modulos'):
            # Creando directorio para almacenar los paquetes en local.
            mkdir('modulos')

        try:
            # Descargando release comprimida.
            response = urllib.request.urlopen(url)

            if response.getcode() != 200:
                print(
                    'ERROR: Verifique que haya escrito la versión '
                    'correctamente.')
                return False

        except urllib.error.HTTPError as e:
            print('ERROR: Versión del paquete incorrecta.')
            return False

        with ZipFile(BytesIO(response.read()), 'r') as zipx:
            # Extrayendo el comprimido
            if exists(f'modulos/{name}'):
                shutil.rmtree(f'modulos/{name}')
                mkdir(f'modulos/{name}')

            zipx.extractall(f'modulos/{name}')
            print('INFO: Paquete instalado.')


if __name__ == '__main__':
    p = paqAdm()
    p.getlibrary(sys.argv[1], sys.argv[2])



import shutil, json
import urllib.request
from io import BytesIO
from os import remove, chdir, mkdir
from os.path import exists
from zipfile import ZipFile


class repoAdm:
    def __init__(self):
        # Definiendo el origen de los repositorios.
        self.url = urllib.request.Request(
            'https://github.com/N3koSempai/LAP_Directory/archive/refs/heads/main.zip')

    def getrepo(self):
        if not exists('repo'):
            # Creando directorio para repo.
            mkdir('repo')

        try:
            # Descargando lista de paquetes en formato zip.
            response = urllib.request.urlopen(self.url)
            data = response.read()

        except Exception as err:
            print(f"ERROR: {err}")

        finally:
            # Verificando respuesta correcta del servidor.
            if response.status == 200:
                with ZipFile(BytesIO(data), 'r') as zipx:
                    # Extrayendo.
                    zipx.extractall('repo/')

                # Ajustando carpetas y borrando temporales
                shutil.move('repo/LAP_Directory-main/main.json', 'repo/main.json')
                shutil.rmtree('repo/LAP_Directory-main')

                return 'INFO: Repositorios actualizados.'
            else:
                raise ("sin respuesta del servidor de repositorios")

    def jsonSearch(self, actual_dir, library):
        if not exists('repo/main.json'):
            self.getrepo()

        # Abriendo archivo repo tipo json previamente descargado.
        with open("repo/main.json", 'r') as repolist:
            repolib = json.loads(repolist.read())

        try:
            # Buscando nombre de la paquete.
            for paquete in repolib.keys():
                if paquete == library:
                    return repolib[paquete]
                else:
                    print("ERROR: Paquete no encontrado.")

        except Exception:
            print("ERROR: Error en busqueda de la paquete")



import sys, os

"""Elimina el prefijo LAP si quieres probarlo sin instalarlo mediante pip, de
lo contrario la importacion no funcionará correctamente."""
from LAP.modulos.repoadm import repoAdm
from LAP.modulos.paqadm import paqAdm
from . import __version__


def mostrar_ayuda():
    print("Uso:")
    print("  LAP <comando> <opciones>\n")
    print("Commandos:")
    print("  LAP actualizar")
    print("    Actualiza la lista de los paquetes instalables.\n")
    print("  LAP instalar <paquete> <version>")
    print(
        "    Instala el paquete solitado, siempre y cuando se encuentre "
        "registrado.\n"
    )
    print("  LAP version")
    print("    Muestra la versión del administrador de paquetes.\n")
    print("  LAP ayuda")
    print("    Muestra ayuda para los comandos.\n")


repo = repoAdm()
paq = paqAdm()
actual_dir = os.getcwd()

if (len(sys.argv) == 1) or (sys.argv[1] == 'ayuda'):
    mostrar_ayuda()

elif sys.argv[1] == 'actualizar':
    res = repo.getrepo()
    print(res)

elif sys.argv[1] == 'instalar':
    if len(sys.argv) > 2:
        # Buscar paquete que corresponda en los repos internos.
        lib_url = repo.jsonSearch(actual_dir, sys.argv[2])

        # Obtener libreria comprimida.
        if lib_url is None:
            exit(1)
        elif len(sys.argv) > 3:
            paq.getlibrary(actual_dir, sys.argv[2], lib_url, sys.argv[3])
        else:
            paq.getlibrary(actual_dir, sys.argv[2], lib_url)

    else:
        print('ERROR: Debe especificar el paquete a instalar.')

elif sys.argv[1] == 'version':
    print(f"LAP {__version__}")

else:
    # Captura ordenes que no corresponden a ninguna válida.
    print(
        "ERROR: Orden no válida, el error puede estar en el argumento "
        f"'{sys.argv[1]}'"
    )
